% MATLAB Code to reproduce the results of article
% "A three-dimensional small deformation theory for 
% electrohydrodynamics of dielectric drops", J. Fluid. Mech (2020)
% by D. Das and D. Saintillan
% email debasishdas@strath.ac.uk or dstn@ucsd.edu for bugs

close all;
clear; clc;

% Electrical parameters
eps0 = 8.8542*10^(-12);

% Medium : Castor oil
eps1 = 5.3*eps0;
sig1 = 4.5*10^(-11);
mu1 = 0.69;

% viscosity ratio
lambda  = 14.1;

% drop radius
a = [0.90 1.05 1.85 2.45]*10^(-3);

% Drop : Silicone oil
eps2 = 3*8.8542*10^(-12);
sig2 = 1.23*10^(-12);
mu2 = mu1*lambda;

gamma = 4.5*10^(-3);

Q = eps2/eps1;
R = sig1/sig2;


eps21 = (eps2 - eps1)/(eps2 + 2*eps1);
sig21 = (sig2 - sig1)/(sig2 + 2*sig1);
tmw = (eps2+2*eps1)/(sig2+2*sig1);
Ec = abs(sqrt(2*mu1/(eps1*tmw*(eps21-sig21))));

EEc = 0.001:0.01:3.5;

% time step
dt = 0.02;
% Number of iterations
nt = 10000;

Lambda1 = 0.3*(19*lambda+16)*(2*lambda+3)/(lambda+1);
Lambda2 = 9/(1 + 2*R)^2;
Lambda3 = 0.6*(2 + 3*lambda)/(1+lambda);
Lambda4 = 3*(19*lambda+16)*(2*lambda+3)/(3*lambda+2);
Lambda5 = (12 + 13*lambda)/(2+3*lambda);

for i = 1:1:length(a)  
    
    for j = 1:1:length(EEc)
        
        E0 = EEc(j)*Ec;
        Ma = 0.5*(eps21-sig21)/EEc(j)^2;
        CaE = a(i)*eps1*E0^2/gamma;
        delta = 0.75*CaE/(1+2*R)^2;
        
        % Number of iterations after which data is saved
        isave = 100; cnt = 0;
        phisave = zeros(length(lambda),nt);
        Dsave = zeros(length(lambda),nt);
        Px = zeros(nt,1);  Py = zeros(nt,1);
        f02 = zeros(nt,1); f22 = zeros(nt,1); f22t = zeros(nt,1);
        
        % Initialize
        it = 1;
        f02(1) = 0; f22(1) = 0; f22t(1) = 0;
        Px(1) = 0.01; Py(1) = -(1-Q)/(Q+2);
        df02 = 1; df22 = 1; df22t = 1;
        dPx = 1; dPy = 1; tt = 0;
        
        for it = 1:1:nt
            
            % save values at t=t_n
            Pxold = Px(it); Pyold = Py(it);
            f02old = f02(it); f22old = f22(it); f22told = f22t(it);
            
            % find the tilt and shape at t=t_n
            phisave(i,it) = 0.5*atan2(f22told,f22old);
            Dsave(i,it) = 3*delta*(f22old*cos(2*phisave(i,it)) ...
                + f22told*sin(2*phisave(i,it)));
            
            tempPx = Pxold;
            tempPy = Pyold;
            tempf02 = f02old;
            tempf22 = f22old;
            tempf22t = f22told;
            
            % --------------
            % First substep
            % --------------
            
            % find dipole
            Px_out = 2*tempPx; Px_in = -tempPx;
            Py_out = 2*tempPy + 1; Py_in = -tempPy + 1;
            
            % find stresses
            pE02 =-0.25*(Px_out^2 + Py_out^2 + (1-2*Q)*(Px_in^2 + Py_in^2));
            pE22 = 0.25*(Px_out^2 - Py_out^2 + (1-2*Q)*(Px_in^2 - Py_in^2));
            pE22t = 0.50*(Px_out*Py_out + (1-2*Q)*Px_in*Py_in);
            
            qE02 = 0.25*(Px_out*Px_in + Py_out*Py_in - Q*(Px_in^2 + Py_in^2));
            qE22 = 0.25*(Px_out*Px_in - Py_out*Py_in - Q*(Px_in^2 - Py_in^2));
            qE22t = 0.25*(Px_out*Py_in + Px_in*Py_out -2*Q*Px_in*Py_in);
            
            % find velocities
            vF02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda3*qE02)/(Ma*Lambda1);
            vF22 =  -(Lambda2*tempf22  - pE22   - Lambda3*qE22  )/(Ma*Lambda1);
            vF22t = -(Lambda2*tempf22t - pE22t  - Lambda3*qE22t )/(Ma*Lambda1);
            
            B02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda5*qE02)/(Ma*Lambda4);
            B22 =  -(Lambda2*tempf22  - pE22   - Lambda5*qE22  )/(Ma*Lambda4);
            B22t = -(Lambda2*tempf22t - pE22t  - Lambda5*qE22t )/(Ma*Lambda4);
            
            % find fluxes
            
            % No straining
            dPx1 = -tempPx - (tempPy + (1-Q)/(Q+2))*tempPx/(2*Ma);
            dPy1 = -tempPy + (1-R)/(1+2*R) + tempPx*tempPx/(2*Ma);
            % Add straining
            dPx1 = dPx1 - (-0.4*vF02 + 2.4*vF22)*tempPx ...
                - 2.4*vF22t*(tempPy + (1-Q)/(Q+2)) - 0.6*B02*tempPx  ...
                + 3.6*(tempPx*B22 + (tempPy + (1-Q)/(Q+2))*B22t);
            
            dPy1 = dPy1 - (-0.4*vF02 - 2.4*vF22)*(tempPy + (1-Q)/(Q+2)) ...
                - 2.4*vF22t*tempPx - 0.6*B02*(tempPy + (1-Q)/(Q+2))  ...
                - 3.6*((tempPy + (1-Q)/(Q+2))*B22 - tempPx*B22t);
            
            df02_1  = vF02/delta;
            df22_1  = -(tempPx/Ma)*tempf22t + vF22/delta;
            df22t_1 = (tempPx/Ma)*tempf22  + vF22t/delta;
            
            % find temp substeps
            
            tempPx = Pxold + 0.5*dPx1*dt;
            tempPy = Pyold + 0.5*dPy1*dt;
            tempf02 = f02old + 0.5*df02_1*dt;
            tempf22 = f22old + 0.5*df22_1*dt;
            tempf22t = f22told + 0.5*df22t_1*dt;
            
            % --------------
            % Second substep
            % --------------
            
            % find dipole
            Px_out = 2*tempPx; Px_in = -tempPx;
            Py_out = 2*tempPy + 1; Py_in = -tempPy + 1;
            
            % find stresses
            pE02 =-0.25*(Px_out^2 + Py_out^2 + (1-2*Q)*(Px_in^2 + Py_in^2));
            pE22 = 0.25*(Px_out^2 - Py_out^2 + (1-2*Q)*(Px_in^2 - Py_in^2));
            pE22t = 0.50*(Px_out*Py_out + (1-2*Q)*Px_in*Py_in);
            
            qE02 = 0.25*(Px_out*Px_in + Py_out*Py_in - Q*(Px_in^2 + Py_in^2));
            qE22 = 0.25*(Px_out*Px_in - Py_out*Py_in - Q*(Px_in^2 - Py_in^2));
            qE22t = 0.25*(Px_out*Py_in + Px_in*Py_out -2*Q*Px_in*Py_in);
            
            % find velocities
            vF02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda3*qE02)/(Ma*Lambda1);
            vF22 =  -(Lambda2*tempf22  - pE22   - Lambda3*qE22  )/(Ma*Lambda1);
            vF22t = -(Lambda2*tempf22t - pE22t  - Lambda3*qE22t )/(Ma*Lambda1);
            
            B02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda5*qE02)/(Ma*Lambda4);
            B22 =  -(Lambda2*tempf22  - pE22   - Lambda5*qE22  )/(Ma*Lambda4);
            B22t = -(Lambda2*tempf22t - pE22t  - Lambda5*qE22t )/(Ma*Lambda4);
            
            % find fluxes
            
            % No straining
            dPx2 = -tempPx - (tempPy + (1-Q)/(Q+2))*tempPx/(2*Ma);
            dPy2 = -tempPy + (1-R)/(1+2*R) + tempPx*tempPx/(2*Ma);
            % Add straining
            dPx2 = dPx2 - (-0.4*vF02 + 2.4*vF22)*tempPx ...
                - 2.4*vF22t*(tempPy + (1-Q)/(Q+2)) - 0.6*B02*tempPx  ...
                + 3.6*(tempPx*B22 + (tempPy + (1-Q)/(Q+2))*B22t);
            
            dPy2 = dPy2 - (-0.4*vF02 - 2.4*vF22)*(tempPy + (1-Q)/(Q+2)) ...
                - 2.4*vF22t*tempPx - 0.6*B02*(tempPy + (1-Q)/(Q+2))  ...
                - 3.6*((tempPy + (1-Q)/(Q+2))*B22 - tempPx*B22t);
            
            df02_2  = vF02/delta;
            df22_2  = -(tempPx/Ma)*tempf22t + vF22/delta;
            df22t_2 = (tempPx/Ma)*tempf22  + vF22t/delta;
            
            % find temp substeps
            
            tempPx = Pxold + 0.5*dPx2*dt;
            tempPy = Pyold + 0.5*dPy2*dt;
            tempf02 = f02old + 0.5*df02_2*dt;
            tempf22 = f22old + 0.5*df22_2*dt;
            tempf22t = f22told + 0.5*df22t_2*dt;
            
            % --------------
            % Third substep
            % --------------
            
            % find dipole
            Px_out = 2*tempPx; Px_in = -tempPx;
            Py_out = 2*tempPy + 1; Py_in = -tempPy + 1;
            
            % find stresses
            pE02 =-0.25*(Px_out^2 + Py_out^2 + (1-2*Q)*(Px_in^2 + Py_in^2));
            pE22 = 0.25*(Px_out^2 - Py_out^2 + (1-2*Q)*(Px_in^2 - Py_in^2));
            pE22t = 0.50*(Px_out*Py_out + (1-2*Q)*Px_in*Py_in);
            
            qE02 = 0.25*(Px_out*Px_in + Py_out*Py_in - Q*(Px_in^2 + Py_in^2));
            qE22 = 0.25*(Px_out*Px_in - Py_out*Py_in - Q*(Px_in^2 - Py_in^2));
            qE22t = 0.25*(Px_out*Py_in + Px_in*Py_out -2*Q*Px_in*Py_in);
            
            % find velocities
            vF02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda3*qE02)/(Ma*Lambda1);
            vF22 =  -(Lambda2*tempf22  - pE22   - Lambda3*qE22  )/(Ma*Lambda1);
            vF22t = -(Lambda2*tempf22t - pE22t  - Lambda3*qE22t )/(Ma*Lambda1);
            
            B02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda5*qE02)/(Ma*Lambda4);
            B22 =  -(Lambda2*tempf22  - pE22   - Lambda5*qE22  )/(Ma*Lambda4);
            B22t = -(Lambda2*tempf22t - pE22t  - Lambda5*qE22t )/(Ma*Lambda4);
            
            % find fluxes
            
            % No straining
            dPx3 = -tempPx - (tempPy + (1-Q)/(Q+2))*tempPx/(2*Ma);
            dPy3 = -tempPy + (1-R)/(1+2*R) + tempPx*tempPx/(2*Ma);
            % Add straining
            dPx3 = dPx3 - (-0.4*vF02 + 2.4*vF22)*tempPx ...
                - 2.4*vF22t*(tempPy + (1-Q)/(Q+2)) - 0.6*B02*tempPx  ...
                + 3.6*(tempPx*B22 + (tempPy + (1-Q)/(Q+2))*B22t);
            
            dPy3 = dPy3 - (-0.4*vF02 - 2.4*vF22)*(tempPy + (1-Q)/(Q+2)) ...
                - 2.4*vF22t*tempPx - 0.6*B02*(tempPy + (1-Q)/(Q+2))  ...
                - 3.6*((tempPy + (1-Q)/(Q+2))*B22 - tempPx*B22t);
            
            df02_3  = vF02/delta;
            df22_3  = -(tempPx/Ma)*tempf22t + vF22/delta;
            df22t_3 = (tempPx/Ma)*tempf22  + vF22t/delta;
            
            % find temp substeps
            
            tempPx = Pxold + 0.5*dPx3*dt;
            tempPy = Pyold + 0.5*dPy3*dt;
            tempf02 = f02old + 0.5*df02_3*dt;
            tempf22 = f22old + 0.5*df22_3*dt;
            tempf22t = f22told + 0.5*df22t_3*dt;
            
            % --------------
            % Fourth substep
            % --------------
            
            % find dipole
            Px_out = 2*tempPx; Px_in = -tempPx;
            Py_out = 2*tempPy + 1; Py_in = -tempPy + 1;
            
            % find stresses
            pE02 =-0.25*(Px_out^2 + Py_out^2 + (1-2*Q)*(Px_in^2 + Py_in^2));
            pE22 = 0.25*(Px_out^2 - Py_out^2 + (1-2*Q)*(Px_in^2 - Py_in^2));
            pE22t = 0.50*(Px_out*Py_out + (1-2*Q)*Px_in*Py_in);
            
            qE02 = 0.25*(Px_out*Px_in + Py_out*Py_in - Q*(Px_in^2 + Py_in^2));
            qE22 = 0.25*(Px_out*Px_in - Py_out*Py_in - Q*(Px_in^2 - Py_in^2));
            qE22t = 0.25*(Px_out*Py_in + Px_in*Py_out -2*Q*Px_in*Py_in);
            
            % find velocities
            vF02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda3*qE02)/(Ma*Lambda1);
            vF22 =  -(Lambda2*tempf22  - pE22   - Lambda3*qE22  )/(Ma*Lambda1);
            vF22t = -(Lambda2*tempf22t - pE22t  - Lambda3*qE22t )/(Ma*Lambda1);
            
            B02 =  -(Lambda2*tempf02  - 2*pE02 + 2*Lambda5*qE02)/(Ma*Lambda4);
            B22 =  -(Lambda2*tempf22  - pE22   - Lambda5*qE22  )/(Ma*Lambda4);
            B22t = -(Lambda2*tempf22t - pE22t  - Lambda5*qE22t )/(Ma*Lambda4);
            
            % find fluxes
            
            % No straining
            dPx4 = -tempPx - (tempPy + (1-Q)/(Q+2))*tempPx/(2*Ma);
            dPy4 = -tempPy + (1-R)/(1+2*R) + tempPx*tempPx/(2*Ma);
            % Add straining
            dPx4 = dPx4 - (-0.4*vF02 + 2.4*vF22)*tempPx ...
                - 2.4*vF22t*(tempPy + (1-Q)/(Q+2)) - 0.6*B02*tempPx  ...
                + 3.6*(tempPx*B22 + (tempPy + (1-Q)/(Q+2))*B22t);
            
            dPy4 = dPy4 - (-0.4*vF02 - 2.4*vF22)*(tempPy + (1-Q)/(Q+2)) ...
                - 2.4*vF22t*tempPx - 0.6*B02*(tempPy + (1-Q)/(Q+2))  ...
                - 3.6*((tempPy + (1-Q)/(Q+2))*B22 - tempPx*B22t);
            
            df02_4  = vF02/delta;
            df22_4  = -(tempPx/Ma)*tempf22t + vF22/delta;
            df22t_4 = (tempPx/Ma)*tempf22  + vF22t/delta;
            
            % Final update
            Px(it+1) = Pxold + dt*(dPx1 + 2*dPx2 + 2*dPx3 + dPx4)/6;
            Py(it+1) = Pyold + dt*(dPy1 + 2*dPy2 + 2*dPy3 + dPy4)/6;
            
            f02(it+1) = f02old + dt*(df02_1 + 2*df02_2 + 2*df02_3 + df02_4)/6;
            f22(it+1) = f22old + dt*(df22_1 + 2*df22_2 + 2*df22_3 + df02_4)/6;
            f22t(it+1) = f22told + dt*(df22t_1+2*df22t_2+2*df22t_3+df22t_4)/6;
            
            dPx =  Px(it+1) - Pxold;
            dPy =  Py(it+1) - Pyold;
            df02 = f02(it+1) - f02old;
            df22 = f22(it+1) - f22old;
            df22t = f22t(it+1) - f22told;
            tt = tt + dt;
        end
        
        phi(i,j) = phisave(i,it-1);
        D1(i,j) = Dsave(i,it-1);
        
    end
   
end

figure(1)
plot(EEc,phi,'Linewidth',2);
lgd = legend('\lambda=0.14');
lgd.FontSize = 14;
grid on;

figure(2)
plot(EEc,D1,'Linewidth',2);
lgd = legend('\lambda=0.14');
lgd.FontSize = 14;
grid on; axis([0 max(EEc)  0 0.3]);